/**
 * Method for running a single process
 * @type FetchResult Result type for the valve
 * @param subqueue Unique key of the subqueue being run
 */
export declare type FetcherProcess<FetchResult, SubqueueKeyType = string | number | symbol> = (subqueue?: SubqueueKeyType) => Promise<FetchResult>;
/**
 * Concurrent queue for a single asynchronous action
 * @type DrainResult Result type when queue is drained
 * @type SubqueueKeyType Error type when queue is drained with an error
 */
export declare class BurstValve<DrainResult, SubqueueKeyType = string | number | symbol> {
    /**
     * Display name for the queue
     */
    readonly displayName: string;
    /**
     * Fetcher for single concurrent running of a function
     */
    private readonly fetcher;
    /**
     * Queue of promise callbacks
     */
    private queue?;
    /**
     * Keyed subqueues of promise callbacks
     */
    private subqueues;
    /**
     * Creates an instance of BurstValve with a custom fetcher
     * @param fetcher Fetcher process for single concurrency process running
     */
    constructor(fetcher: FetcherProcess<DrainResult, SubqueueKeyType>);
    /**
     * Creates an instance of BurstValve with a custom display name and fetcher
     * @param displayName Name for the valve
     * @param fetcher Fetcher process for single concurrency process running
     */
    constructor(displayName: string, fetcher: FetcherProcess<DrainResult, SubqueueKeyType>);
    /**
     * Determines if queue (or subqueue) has an active action being taken
     * @param subqueue Name of queue to check activity. For non-global queues
     */
    isActive(subqueue?: SubqueueKeyType): boolean;
    /**
     * Leverages the current valve to only have a single running process of a function
     * @param subqueue Name of the subqueue
     */
    fetch(subqueue?: SubqueueKeyType): Promise<DrainResult>;
    /**
     * Returns the active queue (or subqueue) after clearing it
     * @param subqueue Subqueue identifier
     */
    private getQueue;
}
//# sourceMappingURL=index.d.ts.map