"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BurstValve = void 0;
/**
 * Concurrent queue for a single asynchronous action
 * @type DrainResult Result type when queue is drained
 * @type SubqueueKeyType Error type when queue is drained with an error
 */
class BurstValve {
    /**
     * Display name for the queue
     */
    displayName;
    /**
     * Fetcher for single concurrent running of a function
     */
    fetcher;
    /**
     * Queue of promise callbacks
     */
    queue;
    /**
     * Keyed subqueues of promise callbacks
     */
    subqueues = {};
    /**
     * Creates an instance of BurstValve with a custom display name and fetcher
     * @param displayName Name for the valve
     * @param fetcher Fetcher process for single concurrency process running
     */
    constructor(displayName, fetcher) {
        if (typeof displayName === "function") {
            fetcher = displayName;
            displayName = "Burst Valve";
        }
        this.displayName = displayName;
        if (typeof fetcher === "function") {
            this.fetcher = fetcher;
        }
        else {
            throw new Error(`Fetcher process not found`);
        }
    }
    /**
     * Determines if queue (or subqueue) has an active action being taken
     * @param subqueue Name of queue to check activity. For non-global queues
     */
    isActive(subqueue) {
        if (subqueue) {
            return this.subqueues[subqueue] ? true : false;
        }
        else {
            return this.queue ? true : false;
        }
    }
    /**
     * Leverages the current valve to only have a single running process of a function
     * @param subqueue Name of the subqueue
     */
    async fetch(subqueue) {
        return new Promise((resolve, reject) => {
            if (subqueue) {
                const list = this.subqueues[subqueue];
                if (list) {
                    return list.push({ resolve, reject });
                }
                else {
                    this.subqueues[subqueue] = [{ resolve, reject }];
                }
            }
            else {
                if (this.queue) {
                    return this.queue.push({ resolve, reject });
                }
                else {
                    this.queue = [{ resolve, reject }];
                }
            }
            this.fetcher(subqueue)
                .then((value) => {
                this.getQueue(subqueue).forEach(({ resolve }) => resolve(value));
            })
                .catch((e) => {
                const error = e instanceof Error
                    ? e
                    : typeof e === "string"
                        ? new Error(e)
                        : new Error(`Unknown Fetcher Error: ${e}`);
                this.getQueue(subqueue).forEach(({ reject }) => reject(error));
            });
        });
    }
    /**
     * Returns the active queue (or subqueue) after clearing it
     * @param subqueue Subqueue identifier
     */
    getQueue(subqueue) {
        let list = [];
        if (subqueue) {
            list = this.subqueues[subqueue] || [];
            if (list) {
                delete this.subqueues[subqueue];
            }
        }
        else if (this.queue) {
            list = this.queue;
            this.queue = undefined;
        }
        return list;
    }
}
exports.BurstValve = BurstValve;
//# sourceMappingURL=index.js.map